@@ -1,11 +1,16 @@
 #!/usr/bin/env python
 
+import logging
 import os
 from errno import ENOENT
 from fuse import FUSE, FuseOSError, Operations, LoggingMixIn
 from stat import S_IFDIR, S_IFREG
 
-class MorosFuse(Operations):
+BLOCK_SIZE = 512
+SUPERBLOCK_ADDR = 4096 * BLOCK_SIZE
+BITMAP_ADDR = SUPERBLOCK_ADDR + 2 * BLOCK_SIZE
+
+class MorosFuse(LoggingMixIn, Operations):
     chmod = None
     chown = None
     create = None
@@ -20,15 +25,20 @@ class MorosFuse(Operations):
     write = None
 
     def __init__(self, path):
+        self.block_size = BLOCK_SIZE
         self.image = open(path, "rb")
-        self.image_offset = 4096
-        self.block_size = 512
-        self.block_count = os.path.getsize(path)
-        addr = self.image_offset * self.block_size
-        self.image.seek(addr)
-        block = self.image.read(self.block_size)
-        assert block[0:8] == b"MOROS FS" # Signature
-        assert block[8] == 1 # Version
+        self.image.seek(SUPERBLOCK_ADDR)
+        superblock = self.image.read(self.block_size)
+        assert superblock[0:8] == b"MOROS FS" # Signature
+        assert superblock[8] == 1 # Version
+        assert self.block_size == 2 << (8 + superblock[9])
+        self.block_count = int.from_bytes(superblock[10:14], "big")
+        self.alloc_count = int.from_bytes(superblock[14:18], "big")
+
+        bs = 8 * self.block_size # number of bits per bitmap block
+        total = self.block_count
+        rest = (total - (BITMAP_ADDR // self.block_size)) * bs // (bs + 1)
+        self.data_addr = BITMAP_ADDR + (rest // bs) * self.block_size
 
     def destroy(self, path):
         self.image.close()
@@ -59,58 +69,43 @@ class MorosFuse(Operations):
     def readdir(self, path, fh):
         files = [".", ".."]
         (_, next_block_addr, _, _, _) = self.__scan(path)
-        while next_block_addr != 0:
-            self.image.seek(next_block_addr)
-            next_block_addr = int.from_bytes(self.image.read(4), "big")
-            offset = 4
-            while offset < self.block_size:
-                kind = int.from_bytes(self.image.read(1), "big")
-                addr = int.from_bytes(self.image.read(4), "big") * self.block_size
-                if addr == 0:
-                    break
-                size = int.from_bytes(self.image.read(4), "big")
-                time = int.from_bytes(self.image.read(8), "big")
-                n = int.from_bytes(self.image.read(1), "big")
-                name = self.image.read(n).decode("utf-8")
-                offset += 1 + 4 + 4 + 8 + 1 + n
-                files.append(name)
+        for (kind, addr, size, time, name) in self.__read(next_block_addr):
+            files.append(name)
         return files
 
+
     def __scan(self, path):
-        dirs = path[1:].split("/")
-        d = dirs.pop(0)
+        next_block_addr = self.data_addr
+        res = (0, next_block_addr, 0, 0, "") # Root dir
+        for d in path[1:].split("/"):
+            if d == "":
+                return res
+            res = (0, 0, 0, 0, "") # Not found
+            for (kind, addr, size, time, name) in self.__read(next_block_addr):
+                if name == d:
+                    res = (kind, addr, size, time, name)
+                    next_block_addr = addr
+                    break
+        return res
 
-        bitmap_area = self.image_offset + 2
-        bs = 8 * self.block_size
-        total = self.block_count // self.block_size
-        rest = (total - bitmap_area) * bs // (bs + 1)
-        data_area = bitmap_area + rest // bs
-
-        next_block_addr = data_area * self.block_size
-        if d == "":
-            return (0, next_block_addr, 0, 0, d)
+    def __read(self, next_block_addr):
         while next_block_addr != 0:
             self.image.seek(next_block_addr)
-            next_block_addr = int.from_bytes(self.image.read(4), "big")
+            next_block_addr = int.from_bytes(self.image.read(4), "big") * self.block_size
             offset = 4
             while offset < self.block_size:
                 kind = int.from_bytes(self.image.read(1), "big")
                 addr = int.from_bytes(self.image.read(4), "big") * self.block_size
-                if addr == 0:
-                    break
                 size = int.from_bytes(self.image.read(4), "big")
                 time = int.from_bytes(self.image.read(8), "big")
                 n = int.from_bytes(self.image.read(1), "big")
-                name = self.image.read(n).decode("utf-8")
-                offset += 1 + 4 + 4 + 1 + n
-                if name == d:
-                    if len(dirs) == 0:
-                        return (kind, addr, size, time, name)
-                    else:
-                        next_block_addr = addr
-                        d = dirs.pop(0)
+                if n == 0:
+                    self.image.seek(-(1 + 4 + 4 + 8 + 1), 1) # Rewind to end of previous entry
                     break
-        return (0, 0, 0, 0, "")
+                name = self.image.read(n).decode("utf-8")
+                offset += 1 + 4 + 4 + 8 + 1 + n
+                if addr > 0:
+                    yield (kind, addr, size, time, name)
 
 if __name__ == '__main__':
     import argparse
@@ -118,4 +113,5 @@ if __name__ == '__main__':
     parser.add_argument('image')
     parser.add_argument('mount')
     args = parser.parse_args()
+    #logging.basicConfig(level=logging.DEBUG)
     fuse = FUSE(MorosFuse(args.image), args.mount, ro=True, foreground=True, allow_other=True)
