@@ -1,12 +1,14 @@
-use crate::api::syscall;
-use crate::sys::fs::{OpenFlag, DeviceType};
 use crate::sys;
+use crate::sys::fs::OpenFlag;
+use crate::api::syscall;
 
 use alloc::format;
 use alloc::string::{String, ToString};
 use alloc::vec::Vec;
 use alloc::vec;
 
+pub use crate::sys::fs::{FileInfo, DeviceType};
+
 pub trait FileIO {
     fn read(&mut self, buf: &mut [u8]) -> Result<usize, ()>;
     fn write(&mut self, buf: &[u8]) -> Result<usize, ()>;
@@ -42,23 +44,20 @@ pub fn realpath(pathname: &str) -> String {
     }
 }
 
-pub fn canonicalize(path: &str) -> Result<String, ()> {
-    match sys::process::env("HOME") {
-        Some(home) => {
-            if path.starts_with('~') {
-                Ok(path.replace("~", &home))
-            } else {
-                Ok(path.to_string())
-            }
-        },
-        None => {
-            Ok(path.to_string())
-        }
+pub fn exists(path: &str) -> bool {
+    syscall::info(path).is_some()
+}
+
+pub fn is_dir(path: &str) -> bool {
+    if let Some(info) = syscall::info(path) {
+        info.is_dir()
+    } else {
+        false
     }
 }
 
-pub fn exists(path: &str) -> bool {
-    syscall::stat(path).is_some()
+pub fn delete(path: &str) -> Result<(), ()> {
+    syscall::delete(path)
 }
 
 pub fn open_file(path: &str) -> Option<usize> {
@@ -89,26 +88,42 @@ pub fn open_device(path: &str) -> Option<usize> {
 pub fn create_device(path: &str, kind: DeviceType) -> Option<usize> {
     let flags = OpenFlag::Create as usize | OpenFlag::Device as usize;
     if let Some(handle) = syscall::open(path, flags) {
-        let buf = [kind as u8; 1];
-        return syscall::write(handle, &buf);
+        syscall::write(handle, &kind.buf());
+        return Some(handle);
     }
     None
 }
 
+pub fn read(path: &str, buf: &mut [u8]) -> Result<usize, ()> {
+    if let Some(info) = syscall::info(&path) {
+        let res = if info.is_device() { open_device(&path) } else { open_file(&path) };
+        if let Some(handle) = res {
+            if let Some(bytes) = syscall::read(handle, buf) {
+                syscall::close(handle);
+                return Ok(bytes);
+            }
+        }
+    }
+    Err(())
+}
+
 pub fn read_to_string(path: &str) -> Result<String, ()> {
-    let buf = read(path)?;
+    let buf = read_to_bytes(path)?;
     Ok(String::from_utf8_lossy(&buf).to_string())
 }
 
-pub fn read(path: &str) -> Result<Vec<u8>, ()> {
-    let path = match canonicalize(path) {
-        Ok(path) => path,
-        Err(_) => return Err(()),
-    };
-    if let Some(stat) = syscall::stat(&path) {
-        let res = if stat.is_device() { open_device(&path) } else { open_file(&path) };
-        if let Some(handle) = res {
-            let mut buf = vec![0; stat.size() as usize];
+pub fn read_to_bytes(path: &str) -> Result<Vec<u8>, ()> {
+    if let Some(info) = syscall::info(&path) {
+        let f = if info.is_device() {
+            open_device(&path)
+        } else if info.is_dir() {
+            open_dir(&path)
+        } else {
+            open_file(&path)
+        };
+        if let Some(handle) = f {
+            let n = info.size() as usize;
+            let mut buf = vec![0; n];
             if let Some(bytes) = syscall::read(handle, &mut buf) {
                 buf.resize(bytes, 0);
                 syscall::close(handle);
@@ -120,10 +135,6 @@ pub fn read(path: &str) -> Result<Vec<u8>, ()> {
 }
 
 pub fn write(path: &str, buf: &[u8]) -> Result<usize, ()> {
-    let path = match canonicalize(path) {
-        Ok(path) => path,
-        Err(_) => return Err(()),
-    };
     if let Some(handle) = create_file(&path) {
         if let Some(bytes) = syscall::write(handle, buf) {
             syscall::close(handle);
@@ -134,12 +145,8 @@ pub fn write(path: &str, buf: &[u8]) -> Result<usize, ()> {
 }
 
 pub fn reopen(path: &str, handle: usize) -> Result<usize, ()> {
-    let path = match canonicalize(path) {
-        Ok(path) => path,
-        Err(_) => return Err(()),
-    };
-    let res = if let Some(stat) = syscall::stat(&path) {
-        if stat.is_device() {
+    let res = if let Some(info) = syscall::info(&path) {
+        if info.is_device() {
             open_device(&path)
         } else {
             open_file(&path)
@@ -155,6 +162,29 @@ pub fn reopen(path: &str, handle: usize) -> Result<usize, ()> {
     Err(())
 }
 
+pub fn read_dir(path: &str) -> Result<Vec<FileInfo>, ()> {
+    if let Some(info) = syscall::info(&path) {
+        if info.is_dir() {
+            if let Ok(buf) = read_to_bytes(path) {
+                let mut res = Vec::new();
+                let mut i = 0;
+                let n = buf.len();
+                while i < n {
+                    let j = i + 14 + buf[i + 13] as usize;
+                    if j > n {
+                        break;
+                    }
+                    let info = FileInfo::from(&buf[i..j]);
+                    res.push(info);
+                    i = j;
+                }
+                return Ok(res);
+            }
+        }
+    }
+    Err(())
+}
+
 #[test_case]
 fn test_file() {
     use crate::sys::fs::{mount_mem, format_mem, dismount};
@@ -168,7 +198,7 @@ fn test_file() {
     assert_eq!(write("/test", &input), Ok(input.len()));
 
     // Read file
-    assert_eq!(read("/test"), Ok(input.to_vec()));
+    assert_eq!(read_to_bytes("/test"), Ok(input.to_vec()));
 
     dismount();
 }
