@@ -36,6 +36,15 @@ macro_rules! eprintln {
     });
 }
 
+#[macro_export]
+macro_rules! error {
+    ($($arg:tt)*) => ({
+        let csi_color = $crate::api::console::Style::color("LightRed");
+        let csi_reset = $crate::api::console::Style::reset();
+        eprintln!("{}Error:{} {}", csi_color, csi_reset, format_args!($($arg)*));
+    });
+}
+
 pub mod console;
 pub mod font;
 pub mod fs;
