@@ -7,30 +7,43 @@ use super::FileType;
 use super::block::LinkedBlock;
 use crate::sys;
 
+use alloc::boxed::Box;
 use alloc::string::String;
 use core::convert::From;
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone)]
 pub struct Dir {
+    parent: Option<Box<Dir>>,
+    name: String,
     addr: u32,
+    size: u32,
+    entry_index: u32,
 }
 
 impl From<DirEntry> for Dir {
     fn from(entry: DirEntry) -> Self {
-        Self { addr: entry.addr() }
+        Self { parent: Some(Box::new(entry.dir())), name: entry.name(), addr: entry.addr(), size: entry.size(), entry_index: 0 }
     }
 }
 
 impl Dir {
     pub fn root() -> Self {
-        Self { addr: SuperBlock::read().data_area() }
+        let name = String::new();
+        let addr = SuperBlock::read().data_area();
+        let mut root = Self { parent: None, name, addr, size: 0, entry_index: 0 };
+        root.update_size();
+        root
+    }
+
+    pub fn is_root(&self) -> bool {
+        self.parent.is_none()
     }
 
     pub fn create(pathname: &str) -> Option<Self> {
         let pathname = realpath(pathname);
         let dirname = dirname(&pathname);
         let filename = filename(&pathname);
-        if let Some(dir) = Dir::open(dirname) {
+        if let Some(mut dir) = Dir::open(dirname) {
             if let Some(dir_entry) = dir.create_dir(filename) {
                 return Some(dir_entry.into());
             }
@@ -81,19 +94,19 @@ impl Dir {
     }
 
     // TODO: return a Result
-    pub fn create_file(&self, name: &str) -> Option<DirEntry> {
+    pub fn create_file(&mut self, name: &str) -> Option<DirEntry> {
         self.create_entry(FileType::File, name)
     }
 
-    pub fn create_dir(&self, name: &str) -> Option<DirEntry> {
+    pub fn create_dir(&mut self, name: &str) -> Option<DirEntry> {
         self.create_entry(FileType::Dir, name)
     }
 
-    pub fn create_device(&self, name: &str) -> Option<DirEntry> {
+    pub fn create_device(&mut self, name: &str) -> Option<DirEntry> {
         self.create_entry(FileType::Device, name)
     }
 
-    fn create_entry(&self, kind: FileType, name: &str) -> Option<DirEntry> {
+    fn create_entry(&mut self, kind: FileType, name: &str) -> Option<DirEntry> {
         if self.find(name).is_some() {
             return None;
         }
@@ -103,14 +116,14 @@ impl Dir {
         while entries.next().is_some() {}
 
         // Allocate a new block for the dir if no space left for adding the new entry
-        let space_left = entries.block.data().len() - entries.block_data_offset();
+        let space_left = entries.block.data().len() - entries.block_offset();
         let entry_len = DirEntry::empty_len() + name.len();
         if entry_len > space_left {
             match entries.block.alloc_next() {
                 None => return None, // Disk is full
                 Some(new_block) => {
                     entries.block = new_block;
-                    entries.block_data_offset = 0;
+                    entries.block_offset = 0;
                 },
             }
         }
@@ -123,7 +136,7 @@ impl Dir {
         let entry_time = sys::clock::realtime() as u64;
         let entry_name = truncate(name, u8::MAX as usize);
         let n = entry_name.len();
-        let i = entries.block_data_offset();
+        let i = entries.block_offset();
         let data = entries.block.data_mut();
 
         data[i] = entry_kind;
@@ -131,11 +144,12 @@ impl Dir {
         data[(i + 5)..(i + 9)].clone_from_slice(&entry_size.to_be_bytes());
         data[(i + 9)..(i + 17)].clone_from_slice(&entry_time.to_be_bytes());
         data[i + 17] = n as u8;
-        data[(i + 18)..(i + 18 + n)].clone_from_slice(&entry_name.as_bytes());
+        data[(i + 18)..(i + 18 + n)].clone_from_slice(entry_name.as_bytes());
 
         entries.block.write();
+        self.update_size();
 
-        Some(DirEntry::new(*self, kind, entry_addr, entry_size, entry_time, &entry_name))
+        Some(DirEntry::new(self.clone(), kind, entry_addr, entry_size, entry_time, &entry_name))
     }
 
     // Deleting an entry is done by setting the entry address to 0
@@ -145,13 +159,14 @@ impl Dir {
         for entry in &mut entries {
             if entry.name() == name {
                 // Zeroing entry addr
-                let i = entries.block_data_offset() - entry.len();
+                let i = entries.block_offset() - entry.len();
                 let data = entries.block.data_mut();
                 data[i + 1] = 0;
                 data[i + 2] = 0;
                 data[i + 3] = 0;
                 data[i + 4] = 0;
                 entries.block.write();
+                self.update_size();
 
                 // Freeing entry blocks
                 let mut entry_block = LinkedBlock::read(entry.addr());
@@ -162,19 +177,18 @@ impl Dir {
                         None => break,
                     }
                 }
-
                 return Ok(());
             }
         }
         Err(())
     }
 
-    pub fn update_entry(&mut self, name: &str, size: u32) {
+    pub fn update_entry(&self, name: &str, size: u32) {
         let time = sys::clock::realtime() as u64;
         let mut entries = self.entries();
         for entry in &mut entries {
             if entry.name() == name {
-                let i = entries.block_data_offset() - entry.len();
+                let i = entries.block_offset() - entry.len();
                 let data = entries.block.data_mut();
                 data[(i + 5)..(i + 9)].clone_from_slice(&size.to_be_bytes());
                 data[(i + 9)..(i + 17)].clone_from_slice(&time.to_be_bytes());
@@ -188,6 +202,10 @@ impl Dir {
         ReadDir::from(self.clone())
     }
 
+    pub fn size(&self) -> usize {
+        self.size as usize
+    }
+
     pub fn delete(pathname: &str) -> Result<(), ()> {
         let pathname = realpath(pathname);
         let dirname = dirname(&pathname);
@@ -198,11 +216,33 @@ impl Dir {
             Err(())
         }
     }
+
+    fn update_size(&mut self) {
+        // The size of a dir is the sum of its dir entries
+        let size: usize = self.entries().map(|e| e.len()).sum();
+        self.size = size as u32;
+        if let Some(dir) = self.parent.clone() {
+            dir.update_entry(&self.name, self.size);
+        }
+    }
 }
 
 impl FileIO for Dir {
-    fn read(&mut self, _buf: &mut [u8]) -> Result<usize, ()> {
-        Err(())
+    fn read(&mut self, buf: &mut [u8]) -> Result<usize, ()> {
+        let mut i = 0;
+        for entry in self.entries().skip(self.entry_index as usize) {
+            let info = entry.info();
+            let bytes = info.as_bytes();
+            let j = i + bytes.len();
+            if j < buf.len() {
+                buf[i..j].copy_from_slice(&bytes);
+                self.entry_index += 1;
+                i = j;
+            } else {
+                break;
+            }
+        }
+        Ok(i)
     }
     fn write(&mut self, _buf: &[u8]) -> Result<usize, ()> {
         Err(())
