@@ -11,15 +11,17 @@ pub struct ReadDir {
     // TODO: make those fields private
     pub dir: Dir,
     pub block: LinkedBlock,
-    pub block_data_offset: usize,
+    pub block_offset: usize,
+    block_index: usize,
 }
 
 impl From<Dir> for ReadDir {
     fn from(dir: Dir) -> Self {
         Self {
-            dir: dir,
+            dir: dir.clone(),
             block: LinkedBlock::read(dir.addr()),
-            block_data_offset: 0,
+            block_offset: 0,
+            block_index: 0,
         }
     }
 }
@@ -28,19 +30,26 @@ macro_rules! read_uint_fn {
     ($name:ident, $type:ident) => {
         fn $name(&mut self) -> $type {
             let data = self.block.data();
-            let a = self.block_data_offset;
+            let a = self.block_offset;
             let b = a + core::mem::size_of::<$type>();
-            self.block_data_offset = b;
+            self.block_offset = b;
             $type::from_be_bytes(data[a..b].try_into().unwrap())
         }
     };
 }
 
 impl ReadDir {
-    pub fn block_data_offset(&self) -> usize {
-        self.block_data_offset
+    /// Total number of bytes read
+    pub fn offset(&self) -> usize {
+        self.block_index * self.block.len() + self.block_offset
     }
 
+    /// Number of bytes read in current block
+    pub fn block_offset(&self) -> usize {
+        self.block_offset
+    }
+
+    /// Address of current block
     pub fn block_addr(&self) -> u32 {
         self.block.addr()
     }
@@ -51,9 +60,9 @@ impl ReadDir {
 
     fn read_utf8_lossy(&mut self, len: usize) -> String {
         let data = self.block.data();
-        let a = self.block_data_offset;
+        let a = self.block_offset;
         let b = a + len;
-        self.block_data_offset = b;
+        self.block_offset = b;
         String::from_utf8_lossy(&data[a..b]).into()
     }
 }
@@ -64,7 +73,7 @@ impl Iterator for ReadDir {
     fn next(&mut self) -> Option<DirEntry> {
         loop {
             loop {
-                let offset = self.block_data_offset; // Backup cursor position
+                let offset = self.block_offset; // Backup cursor position
 
                 // Switch to next block if no space left for another entry
                 if offset >= self.block.len() - DirEntry::empty_len() {
@@ -76,7 +85,7 @@ impl Iterator for ReadDir {
                     1 => FileType::File,
                     2 => FileType::Device,
                     _ => {
-                        self.block_data_offset = offset; // Rewind the cursor
+                        self.block_offset = offset; // Rewind the cursor
                         break;
                     },
                 };
@@ -86,8 +95,8 @@ impl Iterator for ReadDir {
                 let entry_time = self.read_u64();
 
                 let n = self.read_u8() as usize;
-                if n == 0 || n >= self.block.len() - self.block_data_offset {
-                    self.block_data_offset = offset; // Rewind the cursor
+                if n == 0 || n >= self.block.len() - self.block_offset {
+                    self.block_offset = offset; // Rewind the cursor
                     break;
                 }
 
@@ -99,13 +108,15 @@ impl Iterator for ReadDir {
                     continue;
                 }
 
-                return Some(DirEntry::new(self.dir, entry_kind, entry_addr, entry_size, entry_time, &entry_name));
+                let dir = self.dir.clone();
+                return Some(DirEntry::new(dir, entry_kind, entry_addr, entry_size, entry_time, &entry_name));
             }
 
             match self.block.next() {
                 Some(next_block) => {
                     self.block = next_block;
-                    self.block_data_offset = 0;
+                    self.block_offset = 0;
+                    self.block_index += 1;
                 }
                 None => break,
             }
