@@ -1,4 +1,5 @@
 use crate::{api, sys, usr};
+use crate::api::console::Style;
 
 use x86_64::instructions::port::Port;
 
@@ -37,31 +38,34 @@ pub fn main(args: &[&str]) -> usr::shell::ExitCode {
     let n = args.len();
     while i < n {
         match args[i] {
-            "--freq" | "-f" => {
+            "-h" | "--help" => {
+                return help();
+            }
+            "-f" | "--freq" => {
                 if i + 1 < n {
                     if let Ok(value) = args[i + 1].parse() {
                         freq = value;
                     } else {
-                        eprintln!("Could not parse freq");
+                        error!("Could not parse freq");
                         return usr::shell::ExitCode::CommandError;
                     }
                     i += 1;
                 } else {
-                    eprintln!("Missing freq");
+                    error!("Missing freq");
                     return usr::shell::ExitCode::CommandError;
                 }
             },
-            "--len" | "-l" => {
+            "-l" | "--len" => {
                 if i + 1 < n {
                     if let Ok(value) = args[i + 1].parse() {
                         len = value;
                     } else {
-                        eprintln!("Could not parse len");
+                        error!("Could not parse len");
                         return usr::shell::ExitCode::CommandError;
                     }
                     i += 1;
                 } else {
-                    eprintln!("Missing len");
+                    error!("Missing len");
                     return usr::shell::ExitCode::CommandError;
                 }
             },
@@ -73,3 +77,15 @@ pub fn main(args: &[&str]) -> usr::shell::ExitCode {
     beep(freq, len / 1000.0);
     usr::shell::ExitCode::CommandSuccessful
 }
+
+fn help() -> usr::shell::ExitCode {
+    let csi_option = Style::color("LightCyan");
+    let csi_title = Style::color("Yellow");
+    let csi_reset = Style::reset();
+    println!("{}Usage:{} beep {}<options>{1}", csi_title, csi_reset, csi_option);
+    println!();
+    println!("{}Options:{}", csi_title, csi_reset);
+    println!("  {0}-f{1},{0} --freq <hertz>{1}          Tone frequency", csi_option, csi_reset);
+    println!("  {0}-l{1},{0} --len <milliseconds>{1}    Tone length", csi_option, csi_reset);
+    usr::shell::ExitCode::CommandSuccessful
+}
