@@ -10,20 +10,15 @@ pub fn main(args: &[&str]) -> usr::shell::ExitCode {
     let source = args[1];
     let dest = args[2];
 
-    if dest.starts_with("/dev") || dest.starts_with("/sys") {
-        eprintln!("Permission denied to write to '{}'", dest);
-        return usr::shell::ExitCode::CommandError;
-    }
-
-    if let Ok(contents) = fs::read(source) {
+    if let Ok(contents) = fs::read_to_bytes(source) {
         if fs::write(dest, &contents).is_ok() {
             usr::shell::ExitCode::CommandSuccessful
         } else {
-            eprintln!("Could not write to '{}'", dest);
+            error!("Could not write to '{}'", dest);
             usr::shell::ExitCode::CommandError
         }
     } else {
-        eprintln!("File not found '{}'", source);
+        error!("File not found '{}'", source);
         usr::shell::ExitCode::CommandError
     }
 }
