@@ -115,7 +115,7 @@ fn parse_str(input: &str) -> IResult<&str, Exp> {
         value("\n", tag("n")),
     )));
     let (input, s) = delimited(char('"'), escaped, char('"'))(input)?;
-    Ok((input, Exp::Str(s.to_string())))
+    Ok((input, Exp::Str(s)))
 }
 
 fn parse_sym(input: &str) -> IResult<&str, Exp> {
@@ -256,6 +256,32 @@ fn default_env<'a>() -> Env<'a> {
         let contents = fs::read_to_string(&path).or(Err(Err::Reason("Could not read file".to_string())))?;
         Ok(Exp::Str(contents))
     }));
+    data.insert("read-bytes".to_string(), Exp::Func(|args: &[Exp]| -> Result<Exp, Err> {
+        ensure_length_eq!(args, 2);
+        let path = string(&args[0])?;
+        let len = float(&args[1])?;
+        let mut buf = vec![0; len as usize];
+        let bytes = fs::read(&path, &mut buf).or(Err(Err::Reason("Could not read file".to_string())))?;
+        buf.resize(bytes, 0);
+        Ok(Exp::List(buf.iter().map(|b| Exp::Num(*b as f64)).collect()))
+    }));
+    data.insert("bytes".to_string(), Exp::Func(|args: &[Exp]| -> Result<Exp, Err> {
+        ensure_length_eq!(args, 1);
+        let s = string(&args[0])?;
+        let buf = s.as_bytes();
+        Ok(Exp::List(buf.iter().map(|b| Exp::Num(*b as f64)).collect()))
+    }));
+    data.insert("str".to_string(), Exp::Func(|args: &[Exp]| -> Result<Exp, Err> {
+        ensure_length_eq!(args, 1);
+        match &args[0] {
+            Exp::List(list) => {
+                let buf = list_of_floats(list)?.iter().map(|b| *b as u8).collect();
+                let s = String::from_utf8(buf).or(Err(Err::Reason("Could not convert to valid UTF-8 string".to_string())))?;
+                Ok(Exp::Str(s))
+            }
+            _ => Err(Err::Reason("Expected arg to be a list".to_string()))
+        }
+    }));
     data.insert("lines".to_string(), Exp::Func(|args: &[Exp]| -> Result<Exp, Err> {
         ensure_length_eq!(args, 1);
         let s = string(&args[0])?;
@@ -299,7 +325,7 @@ fn list_of_symbols(form: &Exp) -> Result<Vec<String>, Err> {
 }
 
 fn list_of_floats(args: &[Exp]) -> Result<Vec<f64>, Err> {
-    args.iter().map(|x| float(x)).collect()
+    args.iter().map(float).collect()
 }
 
 fn float(exp: &Exp) -> Result<f64, Err> {
@@ -432,6 +458,14 @@ fn eval_mapcar_args(args: &[Exp], env: &mut Env) -> Result<Exp, Err> {
     }
 }
 
+fn eval_progn_args(args: &[Exp], env: &mut Env) -> Result<Exp, Err> {
+    let mut res = Ok(Exp::List(vec![]));
+    for arg in args {
+        res = Ok(eval(&arg, env)?);
+    }
+    res
+}
+
 fn eval_load_args(args: &[Exp], env: &mut Env) -> Result<Exp, Err> {
     ensure_length_eq!(args, 1);
     let path = string(&args[0])?;
@@ -444,7 +478,7 @@ fn eval_load_args(args: &[Exp], env: &mut Env) -> Result<Exp, Err> {
         }
         code = rest;
     }
-    return Ok(Exp::Bool(true));
+    Ok(Exp::Bool(true))
 }
 
 fn eval_built_in_form(exp: &Exp, args: &[Exp], env: &mut Env) -> Option<Result<Exp, Err>> {
@@ -466,6 +500,7 @@ fn eval_built_in_form(exp: &Exp, args: &[Exp], env: &mut Env) -> Option<Result<E
 
                 "defun" | "defn" => Some(eval_defun_args(args, env)),
                 "mapcar" | "map" => Some(eval_mapcar_args(args, env)),
+                "progn" | "do"   => Some(eval_progn_args(args, env)),
                 "load"           => Some(eval_load_args(args, env)),
                 _                => None,
             }
@@ -644,7 +679,7 @@ pub fn main(args: &[&str]) -> usr::shell::ExitCode {
                 }
                 usr::shell::ExitCode::CommandSuccessful
             } else {
-                eprintln!("File not found '{}'", pathname);
+                error!("File not found '{}'", pathname);
                 usr::shell::ExitCode::CommandError
             }
         },
