@@ -21,17 +21,16 @@ pub mod host;
 pub mod http;
 pub mod httpd;
 pub mod install;
-pub mod ip;
 pub mod keyboard;
 pub mod lisp;
 pub mod list;
-pub mod mem;
+pub mod memory;
 pub mod net;
 pub mod pci;
+pub mod pow;
 pub mod print;
 pub mod r#move;
 pub mod read;
-pub mod route;
 pub mod shell;
 pub mod sleep;
 pub mod tcp;
