@@ -8,11 +8,11 @@ use alloc::vec::Vec;
 use alloc::string::String;
 
 // TODO: Scan /bin
-const AUTOCOMPLETE_COMMANDS: [&str; 38] = [
-    "base64", "calc", "clear", "colors", "copy", "date", "delete", "dhcp", "disk", "edit", "env",
-    "exit", "geotime", "goto", "halt", "help", "hex", "host", "http", "httpd", "install", "ip",
-    "keyboard", "lisp", "list", "memory", "move", "net", "pci", "print", "read", "route", "shell",
-    "sleep", "tcp", "user", "vga", "write"
+const AUTOCOMPLETE_COMMANDS: [&str; 37] = [
+    "2048", "base64", "calc", "clear", "colors", "copy", "date", "delete", "dhcp", "disk", "edit",
+    "env", "exit", "geotime", "goto", "halt", "help", "hex", "host", "http", "httpd", "install",
+    "keyboard", "lisp", "list", "memory", "move", "net", "pci", "print", "read", "shell", "sleep",
+    "tcp", "user", "vga", "write"
 ];
 
 #[repr(u8)]
@@ -29,7 +29,7 @@ fn shell_completer(line: &str) -> Vec<String> {
 
     let args = split_args(line);
     let i = args.len() - 1;
-    if args.len() == 1 { // Autocomplete command
+    if args.len() == 1 && !args[0].starts_with("/") { // Autocomplete command
         for &cmd in &AUTOCOMPLETE_COMMANDS {
             if let Some(entry) = cmd.strip_prefix(args[i]) {
                 entries.push(entry.into());
@@ -40,11 +40,11 @@ fn shell_completer(line: &str) -> Vec<String> {
         let dirname = fs::dirname(&pathname);
         let filename = fs::filename(&pathname);
         let sep = if dirname.ends_with('/') { "" } else { "/" };
-        if let Some(dir) = sys::fs::Dir::open(dirname) {
-            for entry in dir.entries() {
-                let name = entry.name();
+        if let Ok(files) = fs::read_dir(dirname) {
+            for file in files {
+                let name = file.name();
                 if name.starts_with(filename) {
-                    let end = if entry.is_dir() { "/" } else { "" };
+                    let end = if file.is_dir() { "/" } else { "" };
                     let path = format!("{}{}{}{}", dirname, sep, name, end);
                     entries.push(path[pathname.len()..].into());
                 }
@@ -99,6 +99,36 @@ pub fn split_args(cmd: &str) -> Vec<&str> {
     args
 }
 
+fn proc(args: &[&str]) -> ExitCode {
+    match args.len() {
+        1 => {
+            ExitCode::CommandSuccessful
+        },
+        2 => {
+            match args[1] {
+                "id" => {
+                    println!("{}", sys::process::id());
+                    ExitCode::CommandSuccessful
+                }
+                "files" => {
+                    for (i, handle) in sys::process::file_handles().iter().enumerate() {
+                        if let Some(resource) = handle {
+                            println!("{}: {:?}", i, resource);
+                        }
+                    }
+                    ExitCode::CommandSuccessful
+                }
+                _ => {
+                    ExitCode::CommandError
+                }
+            }
+        },
+        _ => {
+            ExitCode::CommandError
+        }
+    }
+}
+
 fn change_dir(args: &[&str]) -> ExitCode {
     match args.len() {
         1 => {
@@ -110,11 +140,11 @@ fn change_dir(args: &[&str]) -> ExitCode {
             if pathname.len() > 1 {
                 pathname = pathname.trim_end_matches('/').into();
             }
-            if sys::fs::Dir::open(&pathname).is_some() {
+            if api::fs::is_dir(&pathname) {
                 sys::process::set_dir(&pathname);
                 ExitCode::CommandSuccessful
             } else {
-                println!("File not found '{}'", pathname);
+                error!("File not found '{}'", pathname);
                 ExitCode::CommandError
             }
         },
@@ -129,6 +159,7 @@ pub fn exec(cmd: &str) -> ExitCode {
 
     // Redirections like `print hello => /tmp/hello`
     // Pipes like `print hello -> write /tmp/hello` or `p hello > w /tmp/hello`
+    let mut is_redirected = false;
     let mut n = args.len();
     let mut i = 0;
     loop {
@@ -161,6 +192,7 @@ pub fn exec(cmd: &str) -> ExitCode {
         }
 
         if is_fat_arrow { // Redirections
+            is_redirected = true;
             if i == n - 1 {
                 println!("Could not parse path for redirection");
                 return ExitCode::CommandError;
@@ -181,7 +213,6 @@ pub fn exec(cmd: &str) -> ExitCode {
 
     let res = match args[0] {
         ""                     => ExitCode::CommandSuccessful,
-        "test"                 => { api::syscall::exit(0); ExitCode::CommandSuccessful },
         "a" | "alias"          => ExitCode::CommandUnknown,
         "b"                    => ExitCode::CommandUnknown,
         "c" | "copy"           => usr::copy::main(&args),
@@ -219,25 +250,25 @@ pub fn exec(cmd: &str) -> ExitCode {
         "halt"                 => usr::halt::main(&args),
         "hex"                  => usr::hex::main(&args),
         "net"                  => usr::net::main(&args),
-        "route"                => usr::route::main(&args),
         "dhcp"                 => usr::dhcp::main(&args),
         "http"                 => usr::http::main(&args),
         "httpd"                => usr::httpd::main(&args),
         "tcp"                  => usr::tcp::main(&args),
         "host"                 => usr::host::main(&args),
         "install"              => usr::install::main(&args),
-        "ip"                   => usr::ip::main(&args),
         "geotime"              => usr::geotime::main(&args),
         "colors"               => usr::colors::main(&args),
         "dsk" | "disk"         => usr::disk::main(&args),
         "user"                 => usr::user::main(&args),
-        "mem" | "memory"       => usr::mem::main(&args),
+        "mem" | "memory"       => usr::memory::main(&args),
         "kb" | "keyboard"      => usr::keyboard::main(&args),
         "lisp"                 => usr::lisp::main(&args),
         "chess"                => usr::chess::main(&args),
         "beep"                 => usr::beep::main(&args),
         "elf"                  => usr::elf::main(&args),
         "pci"                  => usr::pci::main(&args),
+        "2048"                 => usr::pow::main(&args),
+        "proc"                 => proc(&args),
         cmd                    => {
             if api::process::spawn(cmd).is_ok() {
                 ExitCode::CommandSuccessful
@@ -248,11 +279,13 @@ pub fn exec(cmd: &str) -> ExitCode {
     };
 
     // TODO: Remove this when redirections are done in spawned process
-    for i in 0..3 {
-        api::fs::reopen("/dev/console", i).ok();
+    if is_redirected {
+        for i in 0..3 {
+            api::fs::reopen("/dev/console", i).ok();
+        }
     }
 
-    return res;
+    res
 }
 
 pub fn run() -> usr::shell::ExitCode {
@@ -332,8 +365,8 @@ fn test_shell() {
     assert_eq!(api::fs::read_to_string("/test"), Ok("test3\n".to_string()));
 
     // Redirect standard error explicitely
-    exec("http 2=> /test");
-    assert_eq!(api::fs::read_to_string("/test"), Ok("Usage: http <host> <path>\n".to_string()));
+    exec("hex /nope 2=> /test");
+    assert!(api::fs::read_to_string("/test").unwrap().contains("File not found '/nope'"));
 
     sys::fs::dismount();
 }
